package at.ac.tuwien.sepm.assignment.individual.persistence;

import at.ac.tuwien.sepm.assignment.individual.entity.Horse;
import at.ac.tuwien.sepm.assignment.individual.entity.Owner;
import at.ac.tuwien.sepm.assignment.individual.exception.NotFoundException;
import at.ac.tuwien.sepm.assignment.individual.exception.PersistenceException;

import java.util.List;
import java.util.Map;

public interface OwnerDao {

    /**
     * @param id of the owner to find.
     * @return the owner with the specified id.
     * @throws NotFoundException will be thrown if the owner could not be found in the database.
     */
    Owner findOneById(Long id) throws NotFoundException;

    /**
     * @return a list of all owners in the system
     * @throws NotFoundException   will be thrown if no owners are present in the database
     */
    List<Owner> getAll() throws NotFoundException;

    /**
     * @param filters to use for filtering the owners
     * @return a list of all owners that fill the criteria
     * @throws NotFoundException wil be thrown if no owners fill the criteria
     */
    List<Owner> getFiltered(Map<String, String> filters) throws NotFoundException;

    /**
     * @param id of the owner whose horses the database should obtain
     * @return a list of all horses that the specified owner owns
     * @throws NotFoundException will be thrown if the owner owns no horses or if the owner could not be found
     */
    List<Horse> getOwnedHorses(Long id) throws NotFoundException;

    /**
     * @param owner that specifies the owner to add
     * @return the newly created owner
     * @throws PersistenceException will be thrown if something goes wrong during the database access.
     */
    Owner addOwner(Owner owner) throws PersistenceException;

    /**
     * @param owner that specifies the new owner values alongside with the id of the owner to update
     * @return the updated owner
     * @throws PersistenceException will be thrown if something goes wrong during the database access.
     * @throws NotFoundException wil be thrown if no owners fill the criteria
     */
    Owner updateOwner(Owner owner) throws PersistenceException, NotFoundException;

    /**
     * @param id of the owner to delete
     * @throws PersistenceException will be thrown if something goes wrong during the database access.
     * @throws NotFoundException    will be thrown if the owner could not be found in the database.
     */
    void deleteOwner(Long id) throws PersistenceException, NotFoundException;
}
